/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-util implementation.

    DTV-util is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-util is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-util.

    DTV-util es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-util se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include "ipv4/sockaddr.h"
#include "ipv6/sockaddr.h"
#include <string>
#include <exception>
#include <typeinfo>

struct sockaddr;

namespace util {
namespace net {

class SockAddr {
public:
	SockAddr( void );
	SockAddr( const SockAddr &sa );
	SockAddr( ipv4::SockAddr &sa );
	SockAddr( ipv6::SockAddr &sa );
	SockAddr( struct sockaddr &sa );
	~SockAddr( void );

	//	Operators
	SockAddr &operator=( const SockAddr &sa );
	SockAddr &operator=( struct sockaddr &sa );
	bool operator==( const SockAddr &other ) const;
	bool operator==( const struct sockaddr &sa ) const;
	operator struct sockaddr() const;

	//	Methods
	static bool isFamilySupported( family::type family );
	family::type family() const;
	std::string asString() const;
	template<typename T,class Operator> inline T visit( Operator oper ) const;
	template<class Operator> inline void visit( Operator oper );

private:
	char _sa[16];
};

template<typename T,class Operator>
inline T SockAddr::visit( Operator oper ) const {
	struct sockaddr *tmp = (struct sockaddr *)_sa;
	
	if (family() == family::ipv4) {
		ipv4::SockAddr sa4( *tmp );
		return oper(sa4);
	}
	else if (family() == family::ipv6) {
		ipv6::SockAddr sa6( *tmp );
		return oper(sa6);
	}
	else {
		throw std::bad_cast();
	}
}

template<class Operator>
inline void SockAddr::visit( Operator oper ) {
	struct sockaddr *tmp = (struct sockaddr *)_sa;

	if (family() == family::ipv4) {
		ipv4::SockAddr sa4( *tmp );
		oper(sa4);
	}
	else if (family() == family::ipv6) {
		ipv6::SockAddr sa6( *tmp );
		oper(sa6);
	}
	else {
		throw std::bad_cast( "Socket family unknown" );
	}
}

}
}

