/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include "../audio/types.h"
#include "../display/display.h"
#include <canvas/rect.h>
#include <boost/signal.hpp>
#include <string>

namespace util {
	class Url;
}

namespace zapper {
namespace player {

typedef boost::signal<void (void)> OnPlayStop;

class MediaPlayer {
public:
	MediaPlayer();
	virtual ~MediaPlayer();

	//	Media
	bool isPlaying() const;
	bool play( const std::string &url );
	void stop();
	void restart();
	void pause( bool pause );

	//	Streams
	virtual int nextAudio();
	virtual int audioCount() const;
	virtual int nextVideo();
	virtual int videoCount() const;
	virtual int nextSubtitle();
	virtual int subtitleCount() const;
	OnPlayStop &onPlayStop();

	//	Video position
	virtual void setFullScreen();
	virtual void resize( int x, int y, int w, int h );

	//	Service specific
	void beginStartService();
	void endStartService();
	void beginStopService();
	void endStopService();		
	bool startPCR( tuner::ID pid );
	bool startStream( bool isVideo, tuner::ID pid, tuner::ID type );
	void stopStream( bool isVideo );	

	//	Getters
	audio::channel::type getAudio();
	display::aspect::type getAspect();
	display::mode::type getResolution();
	const std::string &url() const;

	//	Volume
	virtual bool mute( bool needMute )=0;
	virtual bool volume( audio::Volume vol )=0;

	//	Display notifications
	virtual void onDisplayChanged( void );

protected:
	virtual bool startPlay( const std::string &url )=0;
	virtual void stopPlay()=0;
	virtual void pausePlay( bool pause );

	//	Aux video position
	const canvas::Rect &rect() const;
	virtual void doResize();

	//	Aux info
	virtual void refreshAudio();
	virtual void refreshAspect();
	virtual void refreshDisplay();
	void setAudio( audio::channel::type );
	void setAspect( display::aspect::type );
	void setDisplay( display::mode::type );

private:
	std::string _url;
	canvas::Rect _rect;
	bool _pause;
	bool _allService;
	audio::channel::type _audio;
	display::aspect::type _aspect;
	display::mode::type _display;
	OnPlayStop _onPlayStop;
	util::Url *_srv;
};

}
}

