/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include "../mediaplayer.h"
#include <mpegparser/demuxer/types.h>
#include <vector>

namespace canvas {
	class System;
}

namespace tuner {
namespace stream {
	class Pipe;
	class Fifo;
}
}

namespace zapper {

class Tuner;

namespace player {
namespace mpeg {

class StreamInfo;
class StreamPlayer;

class ServicePlayer : public player::MediaPlayer {
public:
	ServicePlayer( Tuner *tuner, player::MediaPlayer *media );
	virtual ~ServicePlayer();

	//	Add elementary stream player
	void addPlayers( canvas::System *sys );

	//	Media
	virtual void setFullScreen();
	virtual void resize( int x, int y, int w, int h );

	virtual int nextAudio();
	virtual int audioCount() const;
	virtual int nextVideo();
	virtual int videoCount() const;
	virtual int nextSubtitle();
	virtual int subtitleCount() const;

	//	Volume
	virtual bool mute( bool needMute );
	virtual bool volume( audio::Volume vol );

	//	Display notifications
	virtual void onDisplayChanged();

protected:
	//	Mpeg specific
	virtual void onStartBegin();
	virtual void onStartEnd();
	virtual void onStopBegin();	
	virtual void onStopEnd();		
	virtual bool startPCR( bool needFilter );
	virtual void stopPCR();

	//	Streams
	virtual tuner::stream::Pipe *getPCRPipe();
	tuner::ID pcrPID() const;

	//	Stream players
	void addPlayer( StreamPlayer *player );
	StreamPlayer *getPlayer( tuner::pes::type pesType ) const;
	virtual StreamPlayer *create( tuner::pes::type pesType, const std::vector<StreamInfo *> &streams );
	
	//	Aux
	void onServiceStarted( tuner::Service *info, bool started );
	void startMedia( tuner::Service *srv );
	void stopMedia();

	//	Getters
	Tuner *tuner() const;
	player::MediaPlayer *media() const;

	//	MediaPlayer specific
	virtual bool startPlay( const std::string &url );
	virtual void stopPlay();
	virtual void refreshAudio();
	virtual void refreshAspect();
	virtual void refreshDisplay();	

private:
	Tuner *_tuner;
	player::MediaPlayer *_media;
	ServiceID _serviceID;
	std::vector<StreamPlayer *> _players;
	tuner::ID _pcrPID;
	bool _pcrStarted;
	bool _mediaStarted;
};

}
}
}

