/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include <mpegparser/demuxer/types.h>
#include <vector>

namespace tuner {
namespace stream {
	class Pipe;
	class Fifo;
}
}

namespace zapper {
namespace player {

class MediaPlayer;
	
namespace mpeg {

class StreamInfo;

class StreamPlayer {
public:
	StreamPlayer( MediaPlayer *mp, tuner::pes::type type );	
	StreamPlayer( MediaPlayer *mp, tuner::pes::type type, const std::vector<StreamInfo *> &streams );
	virtual ~StreamPlayer( void );

	bool initialize( Tuner *tuner );
	void finalize();

	void start();
	void stop( bool clean );
	
	int next();
	int count() const;
	tuner::pes::type type() const;
	bool collectStream( const tuner::ElementaryInfo &info );

protected:
	virtual bool start( const Stream *sInfo );
	virtual void stop( const Stream *sInfo );
	virtual tuner::stream::Pipe *pipe() const;
	virtual int first() const;	
	virtual bool supportOff() const;
	void start( int id );	

	StreamInfo *getStreamInfo( tuner::ID type ) const;
	void addStream( StreamInfo *info );
	const Stream *getStream( int id ) const;
	int next( int pos, int factor, int size, bool supportOff ) const;
	Tuner *tuner() const;
	
private:
	MediaPlayer *_mp;	
	tuner::pes::type _pesType;
	int _current;
	Tuner *_tuner;
	std::vector<StreamInfo *> _supported;
	std::vector<Stream> _streams;
};

}
}
}
	
