/*******************************************************************************

  Copyright (C) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata

********************************************************************************

  This file is part of DTV-zapper implementation.

    DTV-zapper is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation, either version 2 of the License.

    DTV-zapper is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
  this program. If not, see <http://www.gnu.org/licenses/>.

********************************************************************************

  Este archivo es parte de la implementación de DTV-zapper.

    DTV-zapper es Software Libre: Ud. puede redistribuirlo y/o modificarlo
  bajo los términos de la Licencia Pública General Reducida GNU como es publicada por la
  Free Software Foundation, según la versión 2 de la licencia.

    DTV-zapper se distribuye esperando que resulte de utilidad, pero SIN NINGUNA
  GARANTÍA; ni siquiera la garantía implícita de COMERCIALIZACIÓN o ADECUACIÓN
  PARA ALGÚN PROPÓSITO PARTICULAR. Para más detalles, revise la Licencia Pública
  General Reducida GNU.

    Ud. debería haber recibido una copia de la Licencia Pública General Reducida GNU
  junto a este programa. Si no, puede verla en <http://www.gnu.org/licenses/>.

*******************************************************************************/

#pragma once

#include "types.h"
#include "../audio/types.h"
#include "../display/display.h"
#include <util/types.h>
#include <string>
#include <vector>

namespace canvas {
	class System;
}

namespace zapper {

class Tuner;

namespace player {

namespace mpeg {
	class ServicePlayer;
}

class MediaPlayer;

class Player {
public:
	explicit Player( canvas::System *sys );
	virtual ~Player();

	//	Initialization (aux)
	bool initialize( Tuner *tuner );
	void finalize();

	//	MediaPlayer methods
	MediaPlayer *create( type::type typeOfService );
	void destroy( MediaPlayer *player );

	//	Display methods
	void setResolution( int width, int height );
	const Resolution &getDisplayResolution() const;
	void setAspectRatio( display::aspect::type vAspect );
	display::aspect::type getDisplayAspect() const;

protected:
	//	Initialization (aux)
	virtual bool doInitialize();
	virtual void doFinalize();

	//	Media player
	virtual mpeg::ServicePlayer *createService( Tuner *tuner ) const;
	virtual MediaPlayer *createAudio() const=0;
	virtual MediaPlayer *createVideo() const=0;

	//	Display notifications
	virtual void onDisplayChanged( void );

	//	Getter
	canvas::System *sys() const;

private:
	canvas::System *_sys;
	Tuner *_tuner;	
	Resolution _dResolution;
	display::aspect::type _dAspect;
	std::vector<MediaPlayer *> _players;
};

}
}

